@@ -52,8 +52,7 @@
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "define.h"
+#include "studio.h"
 #if defined(HasTIFF)
 #if defined(HAVE_TIFFCONF_H)
 #include "tiffconf.h"
@@ -74,6 +73,7 @@ static ExceptionInfo
   Forward declarations.
 */
 static unsigned int
+  WritePTIFImage(const ImageInfo *,Image *),
   WriteTIFFImage(const ImageInfo *,Image *);
 
 /*
@@ -166,7 +166,7 @@ static unsigned int ReadColorProfile(char *text,long int length,Image *image)
     }
   image->color_profile.info=(unsigned char *) AcquireMemory(length);
   if (image->color_profile.info == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitWarning,"Memory allocation failed",
+    ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
       image->filename);
   image->color_profile.length=length;
   (void) memcpy(image->color_profile.info,p,length);
@@ -198,7 +198,7 @@ static unsigned int ReadNewsProfile(char *text,long int length,Image *image,
       length*=4;
       image->iptc_profile.info=(unsigned char *) AcquireMemory(length);
       if (image->iptc_profile.info == (unsigned char *) NULL)
-        ThrowBinaryException(ResourceLimitWarning,"Memory allocation failed",
+        ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
           image->filename);
       image->iptc_profile.length=length;
       (void) memcpy(image->iptc_profile.info,p,length);
@@ -240,7 +240,7 @@ static unsigned int ReadNewsProfile(char *text,long int length,Image *image,
 #endif
   image->iptc_profile.info=(unsigned char *) AcquireMemory(length);
   if (image->iptc_profile.info == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitWarning,"Memory allocation failed",
+    ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
       image->filename);
   image->iptc_profile.length=length;
   (void) memcpy(image->iptc_profile.info,p,length);
@@ -311,6 +311,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
   char
+    filename[MaxTextExtent],
     *text;
 
   float
@@ -378,15 +379,37 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryType,exception);
   if (status == False)
-    ThrowReaderException(FileOpenWarning,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"Unable to open file",image);
   tiff_exception=exception;
   (void) TIFFSetErrorHandler((TIFFErrorHandler) TIFFErrors);
   (void) TIFFSetWarningHandler((TIFFErrorHandler) TIFFWarnings);
-  tiff=TIFFClientOpen(image->filename,ReadBinaryUnbufferedType,
-    (thandle_t) image,TIFFReadBlob,TIFFWriteBlob,TIFFSeekBlob,TIFFCloseBlob,
-    TIFFGetBlobSize,TIFFMapBlob,TIFFUnmapBlob);
+  if ((image->blob->file != stdin) && !image->blob->pipet)
+    tiff=TIFFClientOpen(image->filename,ReadBinaryUnbufferedType,
+      (thandle_t) image,TIFFReadBlob,TIFFWriteBlob,TIFFSeekBlob,TIFFCloseBlob,
+      TIFFGetBlobSize,TIFFMapBlob,TIFFUnmapBlob);
+  else
+    {
+      FILE
+        *file;
+
+      int
+        c;
+
+      TemporaryFilename(filename);
+      file=fopen(filename,WriteBinaryType);
+      if (file == (FILE *) NULL)
+        ThrowReaderException(FileOpenWarning,"Unable to write file",image);
+      for (c=ReadBlobByte(image); c != EOF; c=ReadBlobByte(image))
+        (void) fputc(c,file);
+      (void) fclose(file);
+      tiff=TIFFOpen(filename,ReadBinaryType);
+    }
   if (tiff == (TIFF *) NULL)
-    ThrowReaderException(FileOpenWarning,"Unable to open file",image);
+    {
+      if ((image->blob->file == stdin) || image->blob->pipet)
+        remove(filename);
+      ThrowReaderException(FileOpenError,"Unable to open file",image)
+    }
   if (image_info->subrange != 0)
     while (image->scene < image_info->subimage)
     {
@@ -396,8 +419,13 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
       image->scene++;
       status=TIFFReadDirectory(tiff);
       if (status == False)
-        ThrowReaderException(CorruptImageWarning,"Unable to read subimage",
-          image);
+        {
+          TIFFClose(tiff);
+          if ((image->blob->file == stdin) || image->blob->pipet)
+            remove(filename);
+          ThrowReaderException(CorruptImageError,"Unable to read subimage",
+            image)
+        }
     }
   do
   {
@@ -487,8 +515,10 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
         if (!AllocateImageColormap(image,image->colors))
           {
             TIFFClose(tiff);
-            ThrowReaderException(ResourceLimitWarning,
-              "Memory allocation failed",image)
+            if ((image->blob->file == stdin) || image->blob->pipet)
+              remove(filename);
+            ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+              image)
           }
       }
     if (units == RESUNIT_INCH)
@@ -501,7 +531,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     if (TIFFGetField(tiff,TIFFTAG_ARTIST,&text) == 1)
       (void) SetImageAttribute(image,"artist",text);
     if (TIFFGetField(tiff,TIFFTAG_DATETIME,&text) == 1)
-      (void) SetImageAttribute(image,"timeStamp",text);
+      (void) SetImageAttribute(image,"timestamp",text);
     if (TIFFGetField(tiff,TIFFTAG_SOFTWARE,&text) == 1)
       (void) SetImageAttribute(image,"software",text);
     if (TIFFGetField(tiff,TIFFTAG_DOCUMENTNAME,&text) == 1)
@@ -555,13 +585,15 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
         */
         packet_size=bits_per_sample > 8 ? 2 : 1;
         quantum_scanline=(unsigned char *) AcquireMemory(packet_size*width);
-        scanline=(unsigned char *) AcquireMemory(2*TIFFScanlineSize(tiff)+4);
+        scanline=(unsigned char *) AcquireMemory(8*TIFFScanlineSize(tiff));
         if ((quantum_scanline == (unsigned char *) NULL) ||
             (scanline == (unsigned char *) NULL))
           {
             TIFFClose(tiff);
-            ThrowReaderException(ResourceLimitWarning,
-              "Memory allocation failed",image)
+            if ((image->blob->file == stdin) || image->blob->pipet)
+              remove(filename);
+            ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+              image)
           }
         /*
           Create colormap.
@@ -574,10 +606,8 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             {
               image->colormap[i].red=(Quantum)
                 ((unsigned long) (MaxRGB*i)/Max(image->colors-1,1));
-              image->colormap[i].green=(Quantum)
-                ((unsigned long) (MaxRGB*i)/Max(image->colors-1,1));
-              image->colormap[i].blue=(Quantum)
-                ((unsigned long) (MaxRGB*i)/Max(image->colors-1,1));
+              image->colormap[i].green=image->colormap[i].red;
+              image->colormap[i].blue=image->colormap[i].red;
             }
             break;
           }
@@ -588,10 +618,8 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             {
               image->colormap[i].red=(Quantum) (MaxRGB-
                 ((unsigned long) (MaxRGB*i)/Max(image->colors-1,1)));
-              image->colormap[i].green=(Quantum) (MaxRGB-
-                ((unsigned long) (MaxRGB*i)/Max(image->colors-1,1)));
-              image->colormap[i].blue=(Quantum) (MaxRGB-
-                ((unsigned long) (MaxRGB*i)/Max(image->colors-1,1)));
+              image->colormap[i].green=image->colormap[i].red;
+              image->colormap[i].blue=image->colormap[i].red;
             }
             break;
           }
@@ -646,8 +674,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
               */
               lsb_first=1;
               if (*(char *) &lsb_first)
-                MSBOrderShort((char *) scanline,
-                  (TIFFScanlineSize(tiff) << 1)+4);
+                MSBOrderShort(scanline,(TIFFScanlineSize(tiff) << 1)+4);
             }
           p=scanline;
           r=quantum_scanline;
@@ -749,16 +776,26 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
         /*
           Convert TIFF image to DirectClass MIFF image.
         */
-        scanline=(unsigned char *) AcquireMemory(2*TIFFScanlineSize(tiff)+4);
+        (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_EXTRASAMPLES,&extra_samples,
+          &sample_info);
+        scanline=(unsigned char *) AcquireMemory(8*TIFFScanlineSize(tiff));
         if (scanline == (unsigned char *) NULL)
           {
             TIFFClose(tiff);
-            ThrowReaderException(ResourceLimitWarning,
-              "Memory allocation failed",image)
+            if ((image->blob->file == stdin) || image->blob->pipet)
+              remove(filename);
+            ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+              image)
           }
-        (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_EXTRASAMPLES,&extra_samples,
-          &sample_info);
         image->matte=extra_samples == 1;
+        if (image->colorspace == CMYKColorspace)
+          {
+            if (samples_per_pixel > 4)
+              image->matte=True;
+          }
+        else
+          if (samples_per_pixel > 3)
+            image->matte=True;
         for (y=0; y < (long) image->rows; y++)
         {
           q=SetImagePixels(image,0,y,image->columns,1);
@@ -775,8 +812,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
               */
               lsb_first=1;
               if (*(char *) &lsb_first)
-                MSBOrderShort((char *) scanline,
-                  (TIFFScanlineSize(tiff) << 1)+4);
+                MSBOrderShort(scanline,8*TIFFScanlineSize(tiff));
             }
           if (bits_per_sample == 4)
             {
@@ -839,8 +875,10 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
         if (pixels == (uint32 *) NULL)
           {
             TIFFClose(tiff);
-            ThrowReaderException(ResourceLimitWarning,
-              "Memory allocation failed",image)
+            if ((image->blob->file == stdin) || image->blob->pipet)
+              remove(filename);
+            ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+              image)
           }
         (void) TIFFReadRGBAImage(tiff,(uint32) image->columns,
           (uint32) image->rows,pixels+image->columns*sizeof(uint32),0);
@@ -897,6 +935,8 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
       }
   } while (status == True);
   TIFFClose(tiff);
+  if ((image->blob->file == stdin) || image->blob->pipet)
+    remove(filename);
   while (image->previous != (Image *) NULL)
     image=image->previous;
   return(image);
@@ -905,8 +945,8 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
 static Image *ReadTIFFImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
-  ThrowException(exception,MissingDelegateWarning,
-    "TIFF library is not available",image_info->filename);
+  ThrowException(exception,MissingDelegateError,"TIFF library is not available",
+    image_info->filename);
   return((Image *) NULL);
 }
 #endif
@@ -950,7 +990,7 @@ ModuleExport void RegisterTIFFImage(void)
 #endif
   entry=SetMagickInfo("PTIF");
   entry->decoder=ReadTIFFImage;
-  entry->encoder=WriteTIFFImage;
+  entry->encoder=WritePTIFImage;
   entry->adjoin=False;
   entry->description=AcquireString("Pyramid encoded TIFF");
   entry->module=AcquireString("TIFF");
@@ -1008,6 +1048,84 @@ ModuleExport void UnregisterTIFFImage(void)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   W r i t e P T I F I m a g e                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  WritePTIFImage() writes an image in the pyrimid-encoded Tagged image file
+%  format.
+%
+%  The format of the WritePTIFImage method is:
+%
+%      unsigned int WritePTIFImage(const ImageInfo *image_info,Image *image)
+%
+%  A description of each parameter follows:
+%
+%    o status:  Method WritePTIFImage return True if the image is written.
+%      False is returned is there is of a memory shortage or if the image
+%      file cannot be opened for writing.
+%
+%    o image_info: Specifies a pointer to an ImageInfo structure.
+%
+%    o image:  A pointer to a Image structure.
+%
+%
+*/
+static unsigned int WritePTIFImage(const ImageInfo *image_info,Image *image)
+{
+  Image
+    *pyramid_image;
+
+  ImageInfo
+    *clone_info;
+
+  unsigned int
+    status;
+
+  /*
+    Create pyramid-encoded TIFF image.
+  */
+  pyramid_image=CloneImage(image,0,0,True,&image->exception);
+  if (pyramid_image == (Image *) NULL)
+    ThrowWriterException(FileOpenError,"Unable to pyramid-encode image",image);
+  do
+  {
+    pyramid_image->next=ResizeImage(image,pyramid_image->columns/2,
+      pyramid_image->rows/2,TriangleFilter,1.0,&image->exception);
+    if (pyramid_image->next == (Image *) NULL)
+      ThrowWriterException(FileOpenError,"Unable to pyramid-encode image",
+        image);
+    pyramid_image->next->previous=pyramid_image;
+    pyramid_image=pyramid_image->next;
+  } while ((pyramid_image->columns > 64) && (pyramid_image->rows > 64));
+  while (pyramid_image->previous != (Image *) NULL)
+    pyramid_image=pyramid_image->previous;
+  /*
+    Write pyramid-encoded TIFF image.
+  */
+  clone_info=CloneImageInfo(image_info);
+  clone_info->adjoin=True;
+  status=WriteTIFFImage(clone_info,pyramid_image);
+  DestroyImageList(pyramid_image);
+  DestroyImageInfo(clone_info);
+  return(status);
+}
+#else
+static unsigned int WritePTIFImage(const ImageInfo *image_info,Image *image)
+{
+  ThrowBinaryException(MissingDelegateError,"TIFF library is not available",
+    image->filename);
+}
+#endif
+
+#if defined(HasTIFF)
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   W r i t e T I F F I m a g e                                               %
 %                                                                             %
 %                                                                             %
@@ -1225,7 +1343,6 @@ static unsigned int WriteTIFFImage(const ImageInfo *image_info,Image *image)
     *scanline;
 
   unsigned int
-    adjoin,
     scene,
     status;
 
@@ -1241,45 +1358,12 @@ static unsigned int WriteTIFFImage(const ImageInfo *image_info,Image *image)
   assert(image->signature == MagickSignature);
   status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenWarning,"Unable to open file",image);
-  adjoin=image_info->adjoin;
-  if (LocaleCompare(image_info->magick,"PTIF") == 0)
-    {
-      Image
-        *clone_image,
-        *pyramid_image;
-
-      /*
-        Pyramid encode TIFF image.
-      */
-      pyramid_image=CloneImage(image,0,0,True,&image->exception);
-      if (pyramid_image == (Image *) NULL)
-        ThrowWriterException(FileOpenWarning,"Unable to pyramid encode image",
-          image);
-      do
-      {
-        clone_image=CloneImage(pyramid_image,0,0,True,&image->exception);
-        if (clone_image == (Image *) NULL)
-          return(False);
-        pyramid_image->next=ZoomImage(clone_image,pyramid_image->columns >> 1,
-          pyramid_image->rows >> 1,&image->exception);
-        DestroyImage(clone_image);
-        if (pyramid_image->next == (Image *) NULL)
-          ThrowWriterException(FileOpenWarning,"Unable to pyramid encode image",
-            image);
-        pyramid_image->next->previous=pyramid_image;
-        pyramid_image=pyramid_image->next;
-      } while ((pyramid_image->columns > 64) && (pyramid_image->rows > 64));
-      image=pyramid_image;
-      while (image->previous != (Image *) NULL)
-        image=image->previous;
-      adjoin=True;
-    }
+    ThrowWriterException(FileOpenError,"Unable to open file",image);
   tiff_exception=(&image->exception);
   (void) TIFFSetErrorHandler((TIFFErrorHandler) TIFFErrors);
   (void) TIFFSetWarningHandler((TIFFErrorHandler) TIFFWarnings);
   (void) strncpy(filename,image->filename,MaxTextExtent-1);
-  if ((image->file == stdout) || image->pipet ||
+  if ((image->blob->file == stdout) || image->blob->pipet ||
       (image->blob->data != (unsigned char *) NULL))
     TemporaryFilename(filename);
   else
@@ -1372,9 +1456,10 @@ static unsigned int WriteTIFFImage(const ImageInfo *image_info,Image *image)
         (void) TIFFSetField(tiff,TIFFTAG_SAMPLESPERPIXEL,3);
         if (image_info->type != TrueColorType)
           {
-            if (IsGrayImage(image,&image->exception))
+            if ((image_info->type != PaletteType) &&
+                IsGrayImage(image,&image->exception))
               {
-               (void) TIFFSetField(tiff,TIFFTAG_SAMPLESPERPIXEL,1);
+                (void) TIFFSetField(tiff,TIFFTAG_SAMPLESPERPIXEL,1);
                 photometric=PHOTOMETRIC_MINISBLACK;
                 if (IsMonochromeImage(image,&image->exception))
                   {
@@ -1509,11 +1594,12 @@ static unsigned int WriteTIFFImage(const ImageInfo *image_info,Image *image)
       WriteNewsProfile(tiff,TIFFTAG_RICHTIFFIPTC,image);
 #endif
 #endif
-    if (adjoin && (GetImageListSize(image) > 1))
+    if (image_info->adjoin && (GetImageListSize(image) > 1))
       {
         (void) TIFFSetField(tiff,TIFFTAG_SUBFILETYPE,FILETYPE_PAGE);
-        (void) TIFFSetField(tiff,TIFFTAG_PAGENUMBER,(unsigned short)
-          image->scene,GetImageListSize(image));
+        if (image->scene != 0)
+          (void) TIFFSetField(tiff,TIFFTAG_PAGENUMBER,(unsigned short)
+            image->scene,GetImageListSize(image));
       }
     attribute=GetImageAttribute(image,"artist");
     if (attribute != (const ImageAttribute *) NULL)
@@ -1547,10 +1633,9 @@ static unsigned int WriteTIFFImage(const ImageInfo *image_info,Image *image)
     /*
       Write image scanlines.
     */
-    scanline=(unsigned char *) AcquireMemory(2*TIFFScanlineSize(tiff)+4);
+    scanline=(unsigned char *) AcquireMemory(8*TIFFScanlineSize(tiff));
     if (scanline == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",
-        image);
+      ThrowWriterException(ResourceLimitError,"Memory allocation failed",image);
     switch (photometric)
     {
       case PHOTOMETRIC_RGB:
@@ -1680,7 +1765,7 @@ static unsigned int WriteTIFFImage(const ImageInfo *image_info,Image *image)
         if ((blue == (unsigned short *) NULL) ||
             (green == (unsigned short *) NULL) ||
             (red == (unsigned short *) NULL))
-          ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",
+          ThrowWriterException(ResourceLimitError,"Memory allocation failed",
             image);
         /*
           Initialize TIFF colormap.
@@ -1712,7 +1797,8 @@ static unsigned int WriteTIFFImage(const ImageInfo *image_info,Image *image)
           byte,
           polarity;
 
-        if (!IsMonochromeImage(image,&image->exception))
+        if ((image_info->type == PaletteType) ||
+            !IsMonochromeImage(image,&image->exception))
           {
             /*
               Convert PseudoClass packets to contiguous grayscale scanlines.
@@ -1793,12 +1879,12 @@ static unsigned int WriteTIFFImage(const ImageInfo *image_info,Image *image)
       break;
     image=GetNextImage(image);
     MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
-  } while (adjoin);
+  } while (image_info->adjoin);
   while (image->previous != (Image *) NULL)
     image=image->previous;
   TIFFClose(tiff);
-  image->status=False;
-  if ((image->file == stdout) || image->pipet ||
+  image->blob->status=False;
+  if ((image->blob->file == stdout) || image->blob->pipet ||
       (image->blob->data != (unsigned char *) NULL))
     {
       FILE
@@ -1812,21 +1898,19 @@ static unsigned int WriteTIFFImage(const ImageInfo *image_info,Image *image)
       */
       file=fopen(filename,ReadBinaryType);
       if (file == (FILE *) NULL)
-        ThrowWriterException(FileOpenWarning,"Unable to open file",image);
+        ThrowWriterException(FileOpenError,"Unable to open file",image);
       for (c=fgetc(file); c != EOF; c=fgetc(file))
         (void) WriteBlobByte(image,c);
       (void) fclose(file);
       (void) remove(filename);
       CloseBlob(image);
     }
-  if (LocaleCompare(image_info->magick,"PTIF") == 0)
-    DestroyImageList(image);
   return(True);
 }
 #else
 static unsigned int WriteTIFFImage(const ImageInfo *image_info,Image *image)
 {
-  ThrowBinaryException(MissingDelegateWarning,"TIFF library is not available",
+  ThrowBinaryException(MissingDelegateError,"TIFF library is not available",
     image->filename);
 }
 #endif
