@@ -8,12 +8,52 @@
 extern "C" {
 #endif
 
+typedef struct _BlobInfo
+{
+  size_t
+    length,
+    extent,
+    quantum;
+
+  unsigned int
+    mapped,
+    eof;
+
+  unsigned char
+    *data;
+
+  off_t
+    offset,
+    size;
+
+  unsigned int
+    exempt,
+    status,
+    pipet,
+    temporary;
+
+  FILE
+    *file;
+
+  int
+    (*fifo)(const Image *,const void *,const size_t);
+
+  SemaphoreInfo
+    *semaphore;
+
+  long
+    reference_count;
+
+  unsigned long
+    signature;
+} BlobInfo;
 
 /*
   Blob methods.
 */
 extern MagickExport BlobInfo
-  *CloneBlobInfo(const BlobInfo *);
+  *CloneBlobInfo(const BlobInfo *),
+  *ReferenceBlob(BlobInfo *);
 
 extern MagickExport char
   *ReadBlobString(Image *,char *);
@@ -65,8 +105,8 @@ extern MagickExport void
   GetBlobInfo(BlobInfo *),
   *ImageToBlob(const ImageInfo *,Image *,size_t *,ExceptionInfo *),
   *MapBlob(int,const MapMode,off_t,size_t),
-  MSBOrderLong(char *,const size_t),
-  MSBOrderShort(char *,const size_t);
+  MSBOrderLong(unsigned char *,const size_t),
+  MSBOrderShort(unsigned char *,const size_t);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
